/*
Qxw is a program to help construct and publish crosswords.

Copyright 2011-2013 Mark Owen; Windows port by Peter Flippant
http://www.quinapalus.com
E-mail: qxw@quinapalus.com

This file is part of Qxw.

Qxw is free software: you can redistribute it and/or modify
it under the terms of version 2 of the GNU General Public License
as published by the Free Software Foundation.

Qxw is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Qxw.  If not, see <http://www.gnu.org/licenses/> or
write to the Free Software Foundation, Inc., 51 Franklin Street,
Fifth Floor, Boston, MA  02110-1301, USA.
*/

// This file based on
// http://divine.fi.muni.cz/darcs/mainline/external/dlfcn-win32/dlfcn.c
// originally licensed as follows:

/*
* dlfcn-win32
* Copyright (c) 2007 Ramiro Polla
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/



#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "pfdlfcn.h"

static char error_buffer[65535];
static char *current_error;

static int copy_string(char *dest, int dest_size, const char *src) {
  int i = 0;

	if(!src && !dest) return 0;
  for(i = 0; i<dest_size-1;i++) {
    if(!src[i]) break;
    else dest[i]=src[i];
		}	
	dest[i]='\0';
	return i;
}

static void save_err_str(const char *str) {
  DWORD dwMessageId;
  DWORD pos;

  dwMessageId = GetLastError( );
	if(dwMessageId == 0) return;

  // Format error message to: "<argument to function that failed>": <Windows localized error message>
  pos  = copy_string(error_buffer,     sizeof(error_buffer),     "\"");
  pos += copy_string(error_buffer+pos, sizeof(error_buffer)-pos, str);
  pos += copy_string(error_buffer+pos, sizeof(error_buffer)-pos, "\": ");
  pos += FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, dwMessageId,
                        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                        error_buffer+pos, sizeof(error_buffer)-pos, NULL);

  if(pos > 1) if(error_buffer[pos-2]=='\r' && error_buffer[pos-1] == '\n') error_buffer[pos-2] = '\0';
  current_error = error_buffer;
	}

static void save_err_ptr_str(const void *ptr) {
	char ptr_buf[19]; // 0x<pointer> up to 64 bits

  sprintf(ptr_buf, "0x%p", ptr);
  save_err_str(ptr_buf);
	}

void *dlopen(const char *file, int mode) {
  HMODULE hModule = NULL;
  UINT uMode;
	current_error = NULL;

  // Do not let Windows display the critical-error-handler message box
  uMode = SetErrorMode(SEM_FAILCRITICALERRORS);

  if(file == 0) save_err_ptr_str(file);
  else {
		hModule = LoadLibrary((LPSTR) file);
    if(!hModule) save_err_str(file);
		}

  // Return to previous state of the error-mode bit flags
  SetErrorMode(uMode);

  return (void *) hModule;
	}

int dlclose(void *handle) {
  HMODULE hModule = (HMODULE) handle;
  BOOL ret;

  current_error = NULL;

  // dlclose's return value is inverted in relation to FreeLibrary's
  ret = !FreeLibrary(hModule);
	if(ret) save_err_ptr_str(handle);
	return (int) ret;
	}

void *dlsym(void *handle, const char *name) {
  HMODULE hModule = (HMODULE) handle;
	FARPROC symbol;

  current_error = NULL;
  symbol = GetProcAddress(hModule, name);
  if(symbol == NULL) save_err_str(name);
  return (void*) symbol;
	}

char *dlerror(void) {
	char *error_pointer = current_error;
  
	current_error = NULL;
	return error_pointer;
	}
