/*
Qxw is a program to help construct and publish crosswords.

This file is part of Qxw.

Qxw is free software: you can redistribute it and/or modify
it under the terms of version 2 of the GNU General Public License
as published by the Free Software Foundation.

Qxw is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Qxw.  If not, see <http://www.gnu.org/licenses/> or
write to the Free Software Foundation, Inc., 51 Franklin Street,
Fifth Floor, Boston, MA  02110-1301, USA.
*/

// This file originally licensed as follows:

// Author:  Hans Dietrich
//          hdietrich2@hotmail.com
//
// Description:
//     XGetopt.cpp implements getopt(), a function to parse command lines.
//
// History
//     Version 1.2 - 2003 May 17
//     - Added Unicode support
//
//     Version 1.1 - 2002 March 10
//     - Added example to XGetopt.cpp module header
//
// This software is released into the public domain.
// You are free to use it in any way you like.
//
// This software is provided "as is" with no expressed
// or implied warranty.  I accept no liability for any
// damage or loss of business that this software may cause.


#include <windows.h>
#include <stdio.h>
#include <tchar.h>

#include "pfgetopt.h"

TCHAR	*optarg;		  // global argument pointer
int		optind = 0; 	// global argv index

int getopt(int argc, TCHAR *argv[], TCHAR *optstring) {
	TCHAR c;
	TCHAR *cp;
	static TCHAR *next = NULL;
	
	if (optind == 0) next = NULL;
	optarg = NULL;

	if (next == NULL || *next == _T('\0')) {
		if (optind == 0) optind++;
		if (optind >= argc || argv[optind][0] != _T('-') || argv[optind][1] == _T('\0')) {
			optarg = NULL;
			if (optind < argc)
				optarg = argv[optind];
			return EOF;
			}

		if (_tcscmp(argv[optind], _T("--")) == 0) {
			optind++;
			optarg = NULL;
			if (optind < argc)
				optarg = argv[optind];
			return EOF;
			}

		next = argv[optind];
		next++;		// skip past -
		optind++;
		}

	c = *next++;
	cp = _tcschr(optstring, c);

	if (cp == NULL || c == _T(':'))	return _T('?');

	cp++;
	if (*cp == _T(':'))	{
		if (*next != _T('\0')) {
			optarg = next;
			next = NULL;
			}
		else if (optind < argc)	{
			optarg = argv[optind];
			optind++;
			}
		else return _T('?');
		}

	return c;
	}
